//#-hidden-code
//
//  Copyright © 2020 Apple Inc. All rights reserved.
//

import PlaygroundSupport
import Setup

PlaygroundPage.current.liveView = PlaygroundContext.liveViewController

//#-end-hidden-code

import SpriteKit
import Canvas
import Scenes
import SwansQuest

PlaygroundContext.liveViewController.backgroundImage = #imageLiteral(resourceName: "LiveViewBackground.png")

// Title Card
let title = TextScene(says: "My Turtle World", wait: 4.0)
title.loadingDescription = "A dystopian adventure for our Turtle, created by me."

/// World type for our Turtle to play around
class TurtleWorld : Scene, SceneAnimating {
    /// Reusable tile graphic
    static let ground = #imageLiteral(resourceName: "CaveBackground-Floor-64.png")
    
    /// Main Hero of the story
    var hero: Hero
    
    override init() {
        hero = Hero()
        hero.enableAccessibility(label: "Our fearless Hero. Bound for greatness, if they can ever escape this terrible place.")
        super.init(size: Scene.sceneSize)
        prepareLayout()
    }
    
    /// Setup world graphics
    func prepareLayout() {
        // Draw a sky background as the scene's base
        let sky = Graphic(shape: .rectangle(width: 1024, height: 1024, cornerRadius: 0.0), color: #colorLiteral(red: 0.00392156862745098, green: 0.7803921568627451, blue: 0.9882352941176471, alpha: 1.0))
        sky.enableAccessibility(label: "A blue feature-less sky, with no clouds. A desolate place.")
        place(sky, at: Point.center)
        
        // Use a tiled graphic to draw the ground
        let base = Sprite(image: Image(with: TurtleWorld.ground), columns: 16, rows: 4)
        base.enableAccessibility(label: "A ground of brown bricks, stacked neatly.")
        place(base, at: Point(x: 0, y: -384))
        
        // Place the Hero in the starting position
        place(hero, at: Point(x: -250.0, y: -192.0))
    }

    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// Infinite runner. The turtle will continue to pace until you stop your code
    func run(direction: Double) {
        // Turtle walks a distance of 500 points
        self.hero.walk(distance: direction * 500.0) {
            // Change direction
            let newDirection = direction * -1.0
            self.hero.turn()
            
            // Keep walking indefinitely
            self.run(direction: newDirection)
        }
    }
    
    // MARK: - SceneAnimating
    
    /// override default scene transition
    var sceneTransition: SKTransition?
    
    /// delegate used to signal as the scene works through its lifecycle
    var animationDelegate: SceneAnimatingDelegate?
    
    /// accessibility description that will be read when scene is first loaded
    var loadingDescription: String? {
        didSet {
            enableAccessibility(label: loadingDescription!)
        }
    }
    
    /// called after the scene transition has completed
    func sceneIsPresented() {
        run(direction: 1.0)
    }
    
    /// called before the scene is about to be dismissed
    func sceneWillDismiss() {
    }
    
}

let world = TurtleWorld()
world.loadingDescription = "We find our hero trapped in a blue box, unable to escape. Can you help them do something useful?"

// transition one scene to the next with `Runner`
var runner = Runner([ title, world ])
runner.start()
